#include "MyScrollHandler.h"
#include <QComboBox>
#include <QSpinBox>
#include <QEvent>
#include <QScrollArea>
#include <QDebug>

//установки фильтра на чайлды определённого типа
template<class ControlType>
static void MyScrollHandler_installHandlerToChilds(MyScrollHandler* _this,QScrollArea* pS)
{
	if(!pS)return;

	//для всех чайлдов pS подходящего типа
	QList<ControlType*> list=pS->widget()->findChildren<ControlType*>(QString(), Qt::FindChildrenRecursively);
	for(typename QList<ControlType*>::iterator it=list.begin(); it!=list.end(); it++)
	{
		ControlType* child=*it;

		//ставим фокус только по клику или тАбу
		child->setFocusPolicy(Qt::StrongFocus);

		//установка обработчика
		child->installEventFilter(_this);
	}
}

MyScrollHandler::MyScrollHandler(QWidget* FilteredWidget):QObject(FilteredWidget)
{
	//ставим фильтр на виджет FilteredWidget

	//ищем на FilteredWidget все QScrollArea, включая вложенные
	QList<QScrollArea*> list=FilteredWidget->findChildren<QScrollArea*>(QString(), Qt::FindChildrenRecursively);
	for(QList<QScrollArea*>::iterator it=list.begin(); it!=list.end(); it++)
	{
		QScrollArea* pS=*it;

		//фильтр на виджет - для отлова прокрутки на пустой области
		pS->widget()->installEventFilter(this);

		//установка фильтров на чайлды нужных классов
		MyScrollHandler_installHandlerToChilds<QComboBox>(this,pS);
		MyScrollHandler_installHandlerToChilds<QSpinBox>(this,pS);
		//...
	}
}

bool MyScrollHandler::eventFilter(QObject* target, QEvent* event)
{
	do
	{
		if(event->type()!= QEvent::Wheel)
		{
			//остальные сообщения нас тут не интересуют
			break;
		}

		if(QWidget* control=dynamic_cast<QWidget*>(target))
		{
			//отлов прокрутки на пустой области
			if(
			   control &&                 //scrollArea->widget()
			   control->parent() &&       //qt_scrollarea_viewport
			   control->parent()->parent()//scrollArea
			   )
			{
				if(dynamic_cast<QScrollArea*>(control->parent()->parent()))
				{
					//то скидываем фокус с чайлда, если таковой имеется,
					//перестановкой фокуса на себя
					control->setFocus();

					//дальше обычная обработка
					break;
				}
			}

			//если крутят колесо на чайлде, который сейчас не в фокусе,
			if(!control->hasFocus())
			{
				//то не разрешаем обработку колеса
				return true;
			}

			//а иначе - обычная обработка
		}
	}
	while(0);

	//обычная обработка
	return QObject::eventFilter(target, event);
}
